import { Command } from '@my-devkit/core';
import { QueryDocumentSnapshot } from 'firebase-admin/firestore';
import { MemoryOption, SupportedRegion } from 'firebase-functions/v2';
import { FirestoreEvent } from 'firebase-functions/v2/firestore';
import { HttpsFunction } from 'firebase-functions/v2/https';
import { Bus } from './bus';
import { ExecutionMode } from './execution-mode-enum';
import { Type } from './interfaces';
export declare class AppFactory {
    private static functionInRegion;
    private static firestore;
    private static auth;
    private static publicPaths;
    private static instanceId;
    private static options;
    private static apiKey;
    static isActive(name: string): boolean;
    static initializeApp(options: AppFactory.Options): void;
    static createHttpsFunction(name: string, module: Type): HttpsFunction;
    static createOnCallGenkit(name: string, flow: any): CallableFunction;
    static createScheduledJob<C extends Command>(name: string, module: Type, schedule: string, timeZone: string, commandType: new () => C): import("firebase-functions/v1").CloudFunction<unknown>;
    static createEventListener(name: string, module: Type, mode?: ExecutionMode): import("firebase-functions/core").CloudFunction<FirestoreEvent<import("firebase-functions/v2/firestore").QueryDocumentSnapshot, Record<string, string>>>;
    private static registerEventPersisterHandler;
    private static registerControllers;
    private static registerController;
    private static registerHandlers;
    private static registerLogger;
    static getFunctionCommandHandler<C extends Command>(bus: Bus, commandType: new () => C): () => any;
    static getFunctionEventHandler(bus: Bus, mode: ExecutionMode): (snapshot: FirestoreEvent<QueryDocumentSnapshot>) => any;
    static get isEmulator(): boolean;
}
export declare namespace AppFactory {
    interface Options {
        region: SupportedRegion;
        timeoutSeconds: number;
        memory: MemoryOption;
        cors: string[] | boolean;
        publicPaths?: string[];
    }
}
