"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Bus = void 0;
const core_1 = require("@my-devkit/core");
const execution_mode_enum_1 = require("./execution-mode-enum");
class Bus {
    constructor() {
        this.commandHandlers = new Map();
        this.eventHandlers = new Map();
    }
    send(command) {
        return __awaiter(this, void 0, void 0, function* () {
            core_1.Logger.info(`Bus.send ${command._type}`, { command: (0, core_1.serialize)(command) });
            const validationErrors = core_1.Validators.validate(command);
            if (validationErrors.length > 0) {
                throw new core_1.DomainError(validationErrors[0].getErrorMessage());
            }
            const handler = this.commandHandlers.get(command._type);
            if (!handler) {
                throw new Error(`No command handler for ${command._type}`);
            }
            core_1.Logger.info(`Run command handler ${handler.name}`);
            return handler(command);
        });
    }
    publish(event_1) {
        return __awaiter(this, arguments, void 0, function* (event, mode = execution_mode_enum_1.ExecutionMode.Synchronous) {
            core_1.Logger.info(`Bus.publish ${event._type} `, { event: (0, core_1.serialize)(event) });
            const handlers = [...this.getEventHandlers(event._type, mode), ...this.getEventHandlers('**', mode)];
            for (const handler of handlers) {
                core_1.Logger.info(`Run event handler ${handler.name}`);
                yield handler(event);
            }
        });
    }
    onCommand(commandType, handler) {
        const commandName = commandType.constructor.name;
        this.commandHandlers.set(commandName, handler);
    }
    onEvent(eventType, handler, mode) {
        this.registerEventHandler(eventType.constructor.name, mode, handler);
    }
    onEachEvent(handler, mode) {
        this.registerEventHandler('**', mode, handler);
    }
    getDictionaryKey(eventType, mode) {
        return `${eventType}_${mode.toString()}`;
    }
    registerEventHandler(type, mode, handler) {
        const key = this.getDictionaryKey(type, mode);
        if (!this.eventHandlers.has(key)) {
            this.eventHandlers.set(key, []);
        }
        this.eventHandlers.get(key).push(handler);
    }
    getEventHandlers(type, mode) {
        const key = this.getDictionaryKey(type, mode);
        return this.eventHandlers.get(key) || [];
    }
}
exports.Bus = Bus;
//# sourceMappingURL=bus.js.map