"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Context = void 0;
const core_1 = require("@my-devkit/core");
const async_hooks_1 = require("async_hooks");
const injector_1 = require("./injector");
class Context {
    static init() {
        this.asyncHook = (0, async_hooks_1.createHook)({
            init: (asyncId, _type, triggerAsyncId) => {
                if (this.store.has(triggerAsyncId)) {
                    this.store.set(asyncId, this.store.get(triggerAsyncId));
                }
            },
            destroy: (asyncId) => {
                this.store.delete(asyncId);
            }
        });
        this.asyncHook.enable();
    }
    static create(bus) {
        this.store.set((0, async_hooks_1.executionAsyncId)(), { injector: new injector_1.Injector(), bus });
    }
    static resolve(type) {
        if (!this.store.has((0, async_hooks_1.executionAsyncId)())) {
            throw new Error(`Create context before resolving an injection!`);
        }
        return this.store.get((0, async_hooks_1.executionAsyncId)()).injector.resolve(type);
    }
    static getTransactionalClients() {
        if (!this.store.has((0, async_hooks_1.executionAsyncId)())) {
            throw new Error(`Create context before getting transactional clients!`);
        }
        return this.store.get((0, async_hooks_1.executionAsyncId)()).injector.getTransactionalClients();
    }
    static get bus() {
        var _a;
        return (_a = this.store.get((0, async_hooks_1.executionAsyncId)())) === null || _a === void 0 ? void 0 : _a.bus;
    }
    static set correlationId(correlationId) {
        if (!this.store.has((0, async_hooks_1.executionAsyncId)())) {
            throw new Error(`Create context before setting correlationId!`);
        }
        this.store.get((0, async_hooks_1.executionAsyncId)()).correlationId = correlationId;
    }
    static get correlationId() {
        var _a;
        return (_a = this.store.get((0, async_hooks_1.executionAsyncId)())) === null || _a === void 0 ? void 0 : _a.correlationId;
    }
    static set eventId(eventId) {
        if (!this.store.has((0, async_hooks_1.executionAsyncId)())) {
            throw new Error(`Create context before setting eventId!`);
        }
        this.store.get((0, async_hooks_1.executionAsyncId)()).eventId = eventId;
    }
    static get eventId() {
        var _a;
        return (_a = this.store.get((0, async_hooks_1.executionAsyncId)())) === null || _a === void 0 ? void 0 : _a.eventId;
    }
    static set user(user) {
        if (!this.store.has((0, async_hooks_1.executionAsyncId)())) {
            throw new Error(`Create context before setting user!`);
        }
        this.store.get((0, async_hooks_1.executionAsyncId)()).user = user;
    }
    static get user() {
        var _a;
        return (_a = this.store.get((0, async_hooks_1.executionAsyncId)())) === null || _a === void 0 ? void 0 : _a.user;
    }
    static destroy() {
        this.store.delete((0, async_hooks_1.executionAsyncId)());
    }
    static size() {
        return this.store.size;
    }
    static logRemainingKeys() {
        core_1.Logger.info(Array.from(this.store.keys()).join(' '));
    }
    static commit() {
        return __awaiter(this, void 0, void 0, function* () {
            core_1.Logger.info(`Commiting transactions...`);
            for (const client of Context.getTransactionalClients()) {
                yield client.save();
            }
        });
    }
}
exports.Context = Context;
Context.store = new Map();
//# sourceMappingURL=context.js.map