"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@my-devkit/core");
const chai_1 = require("chai");
const reflect_1 = require("../../reflect");
const command_handler_1 = require("./command-handler");
describe('Given a CommandHandler decorator', () => {
    describe('When decorator is called', () => {
        describe('And decorated method has no parameter', () => {
            it('Then we should retrieve information via reflect', () => __awaiter(void 0, void 0, void 0, function* () {
                try {
                    class TestClass {
                        create() {
                            return `create`;
                        }
                    }
                    __decorate([
                        (0, command_handler_1.CommandHandler)(),
                        __metadata("design:type", Function),
                        __metadata("design:paramtypes", []),
                        __metadata("design:returntype", String)
                    ], TestClass.prototype, "create", null);
                    (0, reflect_1.reflect)(TestClass).getHandlerConfiguration();
                }
                catch (error) {
                    (0, chai_1.expect)(error.message).equal('TestClass.create should have one argument!');
                }
            }));
        });
        describe('And decorated method has more than 1 parameter', () => {
            it('Then we should retrieve information via reflect', () => __awaiter(void 0, void 0, void 0, function* () {
                try {
                    class TestClass {
                        create(param1, param2) {
                            return `create ${param1} / ${param2}`;
                        }
                    }
                    __decorate([
                        (0, command_handler_1.CommandHandler)(),
                        __metadata("design:type", Function),
                        __metadata("design:paramtypes", [String, String]),
                        __metadata("design:returntype", String)
                    ], TestClass.prototype, "create", null);
                    (0, reflect_1.reflect)(TestClass).getHandlerConfiguration();
                }
                catch (error) {
                    (0, chai_1.expect)(error.message).equal('TestClass.create should have only one argument!');
                }
            }));
        });
        describe('And decorated method parameter is not a command', () => {
            it('Then we should retrieve information via reflect', () => __awaiter(void 0, void 0, void 0, function* () {
                try {
                    class FakeCommand {
                    }
                    class TestClass {
                        create(fakeCommand) {
                            return `create ${fakeCommand}`;
                        }
                    }
                    __decorate([
                        (0, command_handler_1.CommandHandler)(),
                        __metadata("design:type", Function),
                        __metadata("design:paramtypes", [FakeCommand]),
                        __metadata("design:returntype", String)
                    ], TestClass.prototype, "create", null);
                    (0, reflect_1.reflect)(TestClass).getHandlerConfiguration();
                }
                catch (error) {
                    (0, chai_1.expect)(error.message).equal('TestClass.create should have a command as argument!');
                }
            }));
        });
        describe('And handler is correctly implemented', () => {
            class CreateCommand extends core_1.Command {
            }
            class UpdateCommand extends core_1.Command {
            }
            class TestClass {
                create(command) {
                    return `create: ${command}`;
                }
                update(command) {
                    return `update: ${command}`;
                }
            }
            __decorate([
                (0, command_handler_1.CommandHandler)(),
                __metadata("design:type", Function),
                __metadata("design:paramtypes", [CreateCommand]),
                __metadata("design:returntype", String)
            ], TestClass.prototype, "create", null);
            __decorate([
                (0, command_handler_1.CommandHandler)(),
                __metadata("design:type", Function),
                __metadata("design:paramtypes", [UpdateCommand]),
                __metadata("design:returntype", String)
            ], TestClass.prototype, "update", null);
            it('Then we should retrieve information via reflect', () => __awaiter(void 0, void 0, void 0, function* () {
                const commandHandlers = Array.from((0, reflect_1.reflect)(TestClass).getHandlerConfiguration().commandHandlers.values());
                (0, chai_1.expect)(commandHandlers.length).equal(2, 'Wrong route count detected');
                (0, chai_1.expect)(commandHandlers[0].methodName).equal('create');
                (0, chai_1.expect)(commandHandlers[0].command.constructor.name).equal('CreateCommand');
                (0, chai_1.expect)(commandHandlers[1].methodName).equal('update');
                (0, chai_1.expect)(commandHandlers[1].command.constructor.name).equal('UpdateCommand');
            }));
        });
    });
});
//# sourceMappingURL=command-handler.spec.js.map