"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@my-devkit/core");
const chai_1 = require("chai");
const execution_mode_enum_1 = require("../../execution-mode-enum");
const reflect_1 = require("../../reflect");
const event_handler_1 = require("./event-handler");
describe('Given a EventHandler decorator', () => {
    describe('When decorator is called', () => {
        describe('And decorated method has no parameter', () => {
            it('Then we should retrieve information via reflect', () => __awaiter(void 0, void 0, void 0, function* () {
                try {
                    class TestClass {
                        created() {
                            return `created`;
                        }
                    }
                    __decorate([
                        (0, event_handler_1.EventHandler)(execution_mode_enum_1.ExecutionMode.Asynchronous),
                        __metadata("design:type", Function),
                        __metadata("design:paramtypes", []),
                        __metadata("design:returntype", String)
                    ], TestClass.prototype, "created", null);
                    (0, reflect_1.reflect)(TestClass).getHandlerConfiguration();
                }
                catch (error) {
                    (0, chai_1.expect)(error.message).equal('TestClass.created should have one argument!');
                }
            }));
        });
        describe('And decorated method has more than 1 parameter', () => {
            it('Then we should retrieve information via reflect', () => __awaiter(void 0, void 0, void 0, function* () {
                try {
                    class TestClass {
                        created(param1, param2) {
                            return `created ${param1} / ${param2}`;
                        }
                    }
                    __decorate([
                        (0, event_handler_1.EventHandler)(execution_mode_enum_1.ExecutionMode.Asynchronous),
                        __metadata("design:type", Function),
                        __metadata("design:paramtypes", [String, String]),
                        __metadata("design:returntype", String)
                    ], TestClass.prototype, "created", null);
                    (0, reflect_1.reflect)(TestClass).getHandlerConfiguration();
                }
                catch (error) {
                    (0, chai_1.expect)(error.message).equal('TestClass.created should have only one argument!');
                }
            }));
        });
        describe('And decorated method parameter is not a command', () => {
            it('Then we should retrieve information via reflect', () => __awaiter(void 0, void 0, void 0, function* () {
                try {
                    class FakeEvent {
                    }
                    class TestClass {
                        created(fakeEvent) {
                            return `created ${fakeEvent}`;
                        }
                    }
                    __decorate([
                        (0, event_handler_1.EventHandler)(execution_mode_enum_1.ExecutionMode.Asynchronous),
                        __metadata("design:type", Function),
                        __metadata("design:paramtypes", [FakeEvent]),
                        __metadata("design:returntype", String)
                    ], TestClass.prototype, "created", null);
                    (0, reflect_1.reflect)(TestClass).getHandlerConfiguration();
                }
                catch (error) {
                    (0, chai_1.expect)(error.message).equal('TestClass.created should have an event as argument!');
                }
            }));
        });
        describe('And handler is correctly implemented', () => {
            class CreatedEvent extends core_1.Event {
            }
            class UpdatedEvent extends core_1.Event {
            }
            class TestClass {
                created(event) {
                    return `created: ${event}`;
                }
                updated(event) {
                    return `updated: ${event}`;
                }
            }
            __decorate([
                (0, event_handler_1.EventHandler)(execution_mode_enum_1.ExecutionMode.Asynchronous),
                __metadata("design:type", Function),
                __metadata("design:paramtypes", [CreatedEvent]),
                __metadata("design:returntype", String)
            ], TestClass.prototype, "created", null);
            __decorate([
                (0, event_handler_1.EventHandler)(execution_mode_enum_1.ExecutionMode.Synchronous),
                __metadata("design:type", Function),
                __metadata("design:paramtypes", [UpdatedEvent]),
                __metadata("design:returntype", String)
            ], TestClass.prototype, "updated", null);
            it('Then we should retrieve information via reflect', () => __awaiter(void 0, void 0, void 0, function* () {
                const eventHandlers = Array.from((0, reflect_1.reflect)(TestClass).getHandlerConfiguration().eventHandlers.values());
                (0, chai_1.expect)(eventHandlers.length).equal(2, 'Wrong route count detected');
                (0, chai_1.expect)(eventHandlers[0].methodName).equal('created');
                (0, chai_1.expect)(eventHandlers[0].executionMode).equal(execution_mode_enum_1.ExecutionMode.Asynchronous);
                (0, chai_1.expect)(eventHandlers[0].event.constructor.name).equal('CreatedEvent');
                (0, chai_1.expect)(eventHandlers[1].methodName).equal('updated');
                (0, chai_1.expect)(eventHandlers[1].executionMode).equal(execution_mode_enum_1.ExecutionMode.Synchronous);
                (0, chai_1.expect)(eventHandlers[1].event.constructor.name).equal('UpdatedEvent');
            }));
        });
    });
});
//# sourceMappingURL=event-handler.spec.js.map