import { Document } from '@my-devkit/core';
import { ITransactionalClient } from './interfaces';
export declare class FirestoreClient implements ITransactionalClient {
    private db;
    private operations;
    save(): Promise<void>;
    _find<T extends Document>(path: string): Promise<T>;
    _findWhere<T extends Document>(collection: string, whereClauses?: [string, FirestoreClient.Operator, any | any[]][], options?: FirestoreClient.QueryOptions): Promise<T>;
    _get<T extends Document>(path: string): Promise<T>;
    _getAll<T extends Document>(collection: string, options?: FirestoreClient.QueryOptions): Promise<T[]>;
    _getWhere<T extends Document>(collection: string, whereClauses?: [string, FirestoreClient.Operator, any | any[]][], options?: FirestoreClient.QueryOptions): Promise<T[]>;
    createDocument<T extends Document>(document: T): void;
    updateDocument<T extends Document>(document: T): void;
    deleteDocument<T extends Document>(document: T): void;
    private getCollectionReference;
    private queryCollection;
    private queryDocument;
    private doesDocumentMatchQuery;
    private isCreateOperation;
    private isUpdateOperation;
    private isDeleteOperation;
}
export declare namespace FirestoreClient {
    interface Operation {
        document: Document;
        action: Action;
        committed: boolean;
    }
    enum Action {
        Create,
        Update,
        Delete
    }
    type Operator = FirebaseFirestore.WhereFilterOp;
    interface QueryOptions {
        orderBy?: string;
        orderByDirection?: FirebaseFirestore.OrderByDirection;
        limit?: number;
    }
}
