"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var FirestoreClient_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.FirestoreClient = void 0;
const core_1 = require("@my-devkit/core");
const firestore_1 = require("firebase-admin/firestore");
const decorators_1 = require("./decorators");
let FirestoreClient = FirestoreClient_1 = class FirestoreClient {
    constructor() {
        this.db = (0, firestore_1.getFirestore)();
        this.operations = new Map();
    }
    save() {
        return __awaiter(this, void 0, void 0, function* () {
            for (const operation of Array.from(this.operations.values())) {
                if (operation.committed === true) {
                    continue;
                }
                if (this.isCreateOperation(operation)) {
                    core_1.Logger.info(`Create document ${operation.document._path}`);
                    yield this.db.doc(operation.document._path).set((0, core_1.serialize)(operation.document));
                }
                else if (this.isUpdateOperation(operation)) {
                    core_1.Logger.info(`Update document ${operation.document._path}`);
                    yield this.db.doc(operation.document._path).set((0, core_1.serialize)(operation.document));
                }
                else if (this.isDeleteOperation(operation)) {
                    core_1.Logger.info(`Delete document ${operation.document._path}`);
                    yield this.db.doc(operation.document._path).delete();
                }
                else {
                    throw new Error(`Not supported action ${operation.action}`);
                }
                operation.committed = true;
            }
        });
    }
    _find(path) {
        return __awaiter(this, void 0, void 0, function* () {
            core_1.Logger.info(`Repository: find ${path}`);
            if (this.operations.has(path)) {
                core_1.Logger.info(`Repository: document returned from transaction`);
                const operation = this.operations.get(path);
                return this.isDeleteOperation(operation) ? null : operation.document;
            }
            return this.queryDocument(path);
        });
    }
    _findWhere(collection_1) {
        return __awaiter(this, arguments, void 0, function* (collection, whereClauses = [], options) {
            const documents = yield this._getWhere(collection, whereClauses, options);
            if (documents.length > 1) {
                throw new Error(`Transactional Repository: Too many documents found (${documents.length})`);
            }
            return documents[0] || null;
        });
    }
    _get(path) {
        return __awaiter(this, void 0, void 0, function* () {
            core_1.Logger.info(`Repository: get ${path}`);
            const result = yield this._find(path);
            if (!result) {
                throw new core_1.NotFoundError(`Repository: document ${path} not found`);
            }
            return result;
        });
    }
    _getAll(collection, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this._getWhere(collection, [], options);
        });
    }
    _getWhere(collection_1) {
        return __awaiter(this, arguments, void 0, function* (collection, whereClauses = [], options) {
            core_1.Logger.info(`Repository: getWhere ${collection} whereClauses: ${JSON.stringify(whereClauses)}`);
            const documentMap = yield this.queryCollection(collection, whereClauses, options);
            for (const operation of Array.from(this.operations.values())) {
                const exists = documentMap.has(operation.document._path);
                switch (operation.action) {
                    case FirestoreClient_1.Action.Create:
                        if (this.doesDocumentMatchQuery(operation.document, collection, whereClauses)) {
                            documentMap.set(operation.document._path, operation.document);
                            core_1.Logger.info(`Repository: Document ${operation.document._path} added to result`);
                        }
                        break;
                    case FirestoreClient_1.Action.Update:
                        if (this.doesDocumentMatchQuery(operation.document, collection, whereClauses)) {
                            core_1.Logger.info(`Repository: Document ${operation.document._path} ${exists ? 'updated from' : 'added to'} result`);
                            documentMap.set(operation.document._path, operation.document);
                        }
                        else if (exists) {
                            documentMap.delete(operation.document._path);
                            core_1.Logger.info(`Repository: Document ${operation.document._path} removed from result`);
                        }
                        break;
                    case FirestoreClient_1.Action.Delete:
                        if (exists) {
                            documentMap.delete(operation.document._path);
                            core_1.Logger.info(`Repository: Document ${operation.document._path} removed from result`);
                        }
                        break;
                    default:
                        throw new Error(`FirestoreClient: Unhandled existing operation ${operation.action} on ${operation.document._path}`);
                }
            }
            core_1.Logger.info(`Repository: ${documentMap.size} documents found`);
            let documents = Array.from(documentMap.values());
            if (options === null || options === void 0 ? void 0 : options.orderBy) {
                documents = (0, core_1._orderBy)(documents, [options.orderBy], [options.orderByDirection || 'asc']);
            }
            if (options === null || options === void 0 ? void 0 : options.limit) {
                documents = documents.slice(0, options.limit);
            }
            return documents;
        });
    }
    createDocument(document) {
        const existingOperation = this.operations.get(document._path);
        if (existingOperation) {
            switch (existingOperation.action) {
                case FirestoreClient_1.Action.Create:
                    throw new Error(`FirestoreClient: can't create twice document ${document._path}`);
                case FirestoreClient_1.Action.Update:
                    throw new Error(`FirestoreClient: can't create already existing document ${document._path}`);
                case FirestoreClient_1.Action.Delete:
                    existingOperation.action = FirestoreClient_1.Action.Update;
                    existingOperation.document = document;
                    break;
                default:
                    throw new Error(`FirestoreClient: Unhandled existing operation ${existingOperation.action} on ${document._path}`);
            }
        }
        else {
            this.operations.set(document._path, { document, action: FirestoreClient_1.Action.Create, committed: false });
        }
    }
    updateDocument(document) {
        const existingOperation = this.operations.get(document._path);
        if (existingOperation) {
            switch (existingOperation.action) {
                case FirestoreClient_1.Action.Create:
                case FirestoreClient_1.Action.Update:
                    existingOperation.document = document;
                    break;
                case FirestoreClient_1.Action.Delete:
                    throw new Error(`FirestoreClient: can't update deleted document ${document._path}`);
                default:
                    throw new Error(`FirestoreClient: Unhandled existing operation ${existingOperation.action} on ${document._path}`);
            }
        }
        else {
            this.operations.set(document._path, { document, action: FirestoreClient_1.Action.Update, committed: false });
        }
    }
    deleteDocument(document) {
        const existingOperation = this.operations.get(document._path);
        if (existingOperation) {
            switch (existingOperation.action) {
                case FirestoreClient_1.Action.Create:
                    this.operations.delete(document._path);
                    break;
                case FirestoreClient_1.Action.Update:
                    existingOperation.document = document;
                    existingOperation.action = FirestoreClient_1.Action.Delete;
                    break;
                case FirestoreClient_1.Action.Delete:
                    throw new Error(`FirestoreClient: can't delete already deleted document ${document._path}`);
                default:
                    throw new Error(`FirestoreClient: Unhandled existing operation ${existingOperation.action} on ${document._path}`);
            }
        }
        else {
            this.operations.set(document._path, { document, action: FirestoreClient_1.Action.Delete, committed: false });
        }
    }
    getCollectionReference(collection) {
        if (collection.startsWith('**/')) {
            return this.db.collectionGroup(collection.replace('**/', ''));
        }
        else {
            return this.db.collection(collection);
        }
    }
    queryCollection(collection_1) {
        return __awaiter(this, arguments, void 0, function* (collection, whereClauses = [], options) {
            let query = this.getCollectionReference(collection);
            whereClauses.forEach(where => {
                query = query.where(where[0], where[1], where[2]);
            });
            if (options === null || options === void 0 ? void 0 : options.orderBy) {
                query = query.orderBy(options.orderBy, options.orderByDirection || 'asc');
            }
            if (options === null || options === void 0 ? void 0 : options.limit) {
                query = query.limit(options.limit);
            }
            const querySnapshot = yield query.get();
            const documents = new Map();
            querySnapshot.forEach(doc => {
                const document = (0, core_1.deserialize)(doc.data());
                documents.set(document._path, document);
            });
            return documents;
        });
    }
    queryDocument(path) {
        return __awaiter(this, void 0, void 0, function* () {
            const doc = yield this.db.doc(path).get();
            const result = doc.data();
            return result ? (0, core_1.deserialize)(result) : null;
        });
    }
    doesDocumentMatchQuery(document, collection, whereClauses) {
        const serializedDocument = (0, core_1.serialize)(document);
        return document._path.startsWith(`${collection}/`) && whereClauses.every(wc => {
            const documentValue = serializedDocument[wc[0]];
            const operator = wc[1];
            const expectedValue = wc[2];
            switch (operator) {
                case '==':
                    return documentValue === expectedValue;
                case '!=':
                    return documentValue !== undefined && documentValue !== expectedValue;
                case '>':
                    return documentValue > expectedValue;
                case '>=':
                    return documentValue >= expectedValue;
                case '<':
                    return documentValue < expectedValue;
                case '<=':
                    return documentValue <= expectedValue;
                case 'array-contains':
                    return Array.isArray(documentValue) && documentValue.includes(expectedValue);
                case 'array-contains-any':
                    return Array.isArray(documentValue) && Array.isArray(expectedValue) && documentValue.some(v => expectedValue.includes(v));
                case 'in':
                    return Array.isArray(expectedValue) && expectedValue.includes(documentValue);
                case 'not-in':
                    return Array.isArray(expectedValue) && !expectedValue.includes(documentValue);
                default:
                    throw new Error(`Operator ${operator} not handled!`);
            }
        });
    }
    isCreateOperation(operation) {
        return operation.action === FirestoreClient_1.Action.Create;
    }
    isUpdateOperation(operation) {
        return operation.action === FirestoreClient_1.Action.Update;
    }
    isDeleteOperation(operation) {
        return operation.action === FirestoreClient_1.Action.Delete;
    }
};
exports.FirestoreClient = FirestoreClient;
__decorate([
    (0, core_1.Cacheable)(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, Array, Object]),
    __metadata("design:returntype", Promise)
], FirestoreClient.prototype, "queryCollection", null);
__decorate([
    (0, core_1.Cacheable)(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", Promise)
], FirestoreClient.prototype, "queryDocument", null);
exports.FirestoreClient = FirestoreClient = FirestoreClient_1 = __decorate([
    (0, decorators_1.TransactionalClient)(),
    (0, decorators_1.Injectable)()
], FirestoreClient);
(function (FirestoreClient) {
    let Action;
    (function (Action) {
        Action[Action["Create"] = 'Create'] = "Create";
        Action[Action["Update"] = 'Update'] = "Update";
        Action[Action["Delete"] = 'Delete'] = "Delete";
    })(Action = FirestoreClient.Action || (FirestoreClient.Action = {}));
})(FirestoreClient || (exports.FirestoreClient = FirestoreClient = {}));
//# sourceMappingURL=firestore-client.js.map