"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HandlerHelper = void 0;
const core_1 = require("@my-devkit/core");
const context_1 = require("./context");
const execution_mode_enum_1 = require("./execution-mode-enum");
const firestore_client_1 = require("./firestore-client");
class HandlerHelper {
    static create(aggregate, callback) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield callback(aggregate);
            this.client.createDocument(aggregate);
            yield this.publishEvents(aggregate);
            return result;
        });
    }
    static update(aggregateOrId, callback) {
        return __awaiter(this, void 0, void 0, function* () {
            const aggregate = yield this.getAggregate(aggregateOrId);
            if (aggregate.events.length > 0) {
                core_1.Logger.warn(`${aggregate._type} ${aggregate.id} has ${aggregate.events.length} events!!`);
            }
            const hashBefore = aggregate.hash;
            const result = yield callback(aggregate);
            if (aggregate.hash !== hashBefore) {
                this.client.updateDocument(aggregate);
                yield this.publishEvents(aggregate);
            }
            else {
                core_1.Logger.debug(`${aggregate._type} ${aggregate.id} didn't change, ${aggregate.events.length} events skipped`);
                aggregate.clearEvents();
            }
            return result;
        });
    }
    static delete(aggregateOrId, callback) {
        return __awaiter(this, void 0, void 0, function* () {
            const aggregate = yield this.getAggregate(aggregateOrId);
            const result = yield callback(aggregate);
            this.client.deleteDocument(aggregate);
            yield this.publishEvents(aggregate);
            return result;
        });
    }
    static get client() {
        return context_1.Context.resolve(firestore_client_1.FirestoreClient);
    }
    static publishEvents(aggregate) {
        return __awaiter(this, void 0, void 0, function* () {
            const events = aggregate.events;
            aggregate.clearEvents();
            for (const event of events) {
                yield context_1.Context.bus.publish(event, execution_mode_enum_1.ExecutionMode.Synchronous);
            }
        });
    }
    static getAggregate(aggregateOrId) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof aggregateOrId === 'string' || aggregateOrId instanceof String) {
                return this.client._find(`/aggregates/${aggregateOrId}`);
            }
            return aggregateOrId;
        });
    }
}
exports.HandlerHelper = HandlerHelper;
//# sourceMappingURL=handler-helper.js.map