"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Injector = void 0;
const reflect_1 = require("./reflect");
class Injector {
    constructor() {
        this.instances = new Map();
        this.transactionalClients = [];
    }
    resolve(target) {
        const injectableId = (0, reflect_1.reflect)(target).getInjectableId();
        if (!injectableId) {
            throw new Error(`${target.name} is not injectable`);
        }
        if (!this.instances.has(injectableId)) {
            const tokens = (0, reflect_1.reflect)(target).getParameters();
            const injections = tokens.map(token => this.resolve(token));
            const instance = new target(...injections);
            this.instances.set(injectableId, instance);
            if ((0, reflect_1.reflect)(target).isTransactionalClient()) {
                this.transactionalClients.push(instance);
            }
        }
        return this.instances.get(injectableId);
    }
    getTransactionalClients() {
        return [...this.transactionalClients];
    }
}
exports.Injector = Injector;
//# sourceMappingURL=injector.js.map