import 'reflect-metadata';
import { Command, Event } from '@my-devkit/core';
import { ExecutionMode } from './execution-mode-enum';
import { Type } from './interfaces';
import { RequestMethod } from './request-method.enum';
import { Server } from './server';
declare class ReflectHelper<T extends ReflectTarget<any>> {
    private target;
    private methodName;
    private readonly HANDLER_META_DATA;
    private readonly CONTROLLER_META_DATA;
    private readonly INJECTABLE_META_DATA;
    private readonly TRANSACTIONAL_CLIENT_META_DATA;
    constructor(target: T, methodName: string | symbol);
    getParameters(): Type[];
    registerInjectable(): void;
    getInjectableId(): string;
    registerTransactionalClient(): void;
    isTransactionalClient(): boolean;
    getControllerConfiguration(): ControllerConfiguration;
    registerController(basePath: string): void;
    registerControllerRoute(route: {
        path: string;
        requestMethod: RequestMethod;
    }): void;
    registerControllerRouteArgumentInjector<T>(injector: RequestArgumentInjector<T>, index: number): void;
    getHandlerConfiguration(): HandlerConfiguration;
    registerCommandHandler<C extends Command>(command: new () => C): void;
    registerEventHandler<E extends Event>(event: new () => E, executionMode: ExecutionMode): void;
    private getControllerRoute;
    private getMetaData;
    private defineMetaData;
    private hasMetaData;
    private get targetConstructor();
}
export interface ControllerConfiguration {
    basePath: string;
    routes: ControllerRoute[];
}
export interface ControllerRoute {
    path: string;
    methodName: string;
    requestMethod: RequestMethod;
    argumentInjectors: RequestArgumentInjector<any>[];
}
export interface CommandHandler {
    methodName: string;
    command: new () => Command;
}
export interface EventHandler {
    methodName: string;
    executionMode: ExecutionMode;
    event: new () => Event;
}
export interface HandlerConfiguration {
    commandHandlers: Map<string, CommandHandler>;
    eventHandlers: Map<string, EventHandler>;
}
export type RequestArgumentInjector<T = any> = (req: Server.Request) => T;
export declare function reflect<T extends ReflectTarget<any>>(target: T, methodName?: string | symbol): ReflectHelper<T>;
type ReflectTarget<T> = T | (new (...args: any[]) => T);
export {};
