"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.reflect = reflect;
require("reflect-metadata");
const core_1 = require("@my-devkit/core");
/* eslint-disable @typescript-eslint/ban-types */
class ReflectHelper {
    constructor(target, methodName) {
        this.target = target;
        this.methodName = methodName;
        this.HANDLER_META_DATA = '__handler__';
        this.CONTROLLER_META_DATA = '__controller__';
        this.INJECTABLE_META_DATA = '__injectable__';
        this.TRANSACTIONAL_CLIENT_META_DATA = '__transactional_client__';
    }
    getParameters() {
        return Reflect.getMetadata('design:paramtypes', this.target, this.methodName) || [];
    }
    registerInjectable() {
        this.defineMetaData(this.INJECTABLE_META_DATA, (0, core_1.guid)());
    }
    getInjectableId() {
        return this.getMetaData(this.INJECTABLE_META_DATA);
    }
    registerTransactionalClient() {
        this.defineMetaData(this.TRANSACTIONAL_CLIENT_META_DATA, core_1.guid);
    }
    isTransactionalClient() {
        return this.hasMetaData(this.TRANSACTIONAL_CLIENT_META_DATA);
    }
    getControllerConfiguration() {
        if (!this.hasMetaData(this.CONTROLLER_META_DATA)) {
            this.defineMetaData(this.CONTROLLER_META_DATA, {
                basePath: null,
                routes: []
            });
        }
        return this.getMetaData(this.CONTROLLER_META_DATA);
    }
    registerController(basePath) {
        this.getControllerConfiguration().basePath = basePath;
    }
    registerControllerRoute(route) {
        this.getControllerRoute().path = route.path;
        this.getControllerRoute().requestMethod = route.requestMethod;
    }
    registerControllerRouteArgumentInjector(injector, index) {
        this.getControllerRoute().argumentInjectors[index] = injector;
    }
    getHandlerConfiguration() {
        if (!this.hasMetaData(this.HANDLER_META_DATA)) {
            this.defineMetaData(this.HANDLER_META_DATA, {
                commandHandlers: new Map(),
                eventHandlers: new Map()
            });
        }
        return this.getMetaData(this.HANDLER_META_DATA);
    }
    registerCommandHandler(command) {
        this.getHandlerConfiguration().commandHandlers.set(this.methodName.toString(), {
            methodName: this.methodName.toString(),
            command
        });
    }
    registerEventHandler(event, executionMode) {
        this.getHandlerConfiguration().eventHandlers.set(this.methodName.toString(), {
            methodName: this.methodName.toString(),
            executionMode,
            event
        });
    }
    getControllerRoute() {
        const config = this.getControllerConfiguration();
        if (!config.routes.find(r => r.methodName === this.methodName.toString())) {
            config.routes.push({
                methodName: this.methodName.toString(),
                path: null,
                requestMethod: null,
                argumentInjectors: []
            });
        }
        return config.routes.find(r => r.methodName === this.methodName.toString());
    }
    getMetaData(key) {
        return Reflect.getMetadata(key, this.targetConstructor);
    }
    defineMetaData(key, data) {
        Reflect.defineMetadata(key, data, this.targetConstructor);
    }
    hasMetaData(key) {
        return Reflect.hasMetadata(key, this.targetConstructor);
    }
    get targetConstructor() {
        if (this.target['prototype']) {
            return this.target['prototype'].constructor;
        }
        if (this.target.constructor) {
            return this.target.constructor;
        }
        return this.target;
    }
}
function reflect(target, methodName) {
    return new ReflectHelper(target, methodName);
}
//# sourceMappingURL=reflect.js.map