"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AuthenticationMiddleware = AuthenticationMiddleware;
const core_1 = require("@my-devkit/core");
const context_1 = require("../../context");
function AuthenticationMiddleware(auth, publicPaths) {
    return (request, _response, next) => __awaiter(this, void 0, void 0, function* () {
        const authorizationHeader = request.headers.authorization ? request.headers.authorization.toString() : null;
        core_1.Logger.info(`${request.method} ${request.path}: Authentication check`);
        if (publicPaths.includes(request.path)) {
            core_1.Logger.info(`${request.path} is a public path, no authorization check`);
            context_1.Context.user = null;
            next();
        }
        else if (request.method === 'OPTIONS') {
            next();
        }
        else if (!authorizationHeader) {
            throw new core_1.UnauthorizedError('No authorization header found');
        }
        else if (authorizationHeader.startsWith('Bearer')) {
            const idToken = authorizationHeader.split('Bearer ')[1];
            try {
                const decodedIdToken = yield auth.verifyIdToken(idToken, true);
                const userId = decodedIdToken.uid;
                context_1.Context.user = yield auth.getUser(userId);
                next();
            }
            catch (error) {
                throw new core_1.UnauthorizedError(`Error while verifying token ${error.message ? error.message : null}`);
            }
        }
        else if (authorizationHeader.startsWith('Basic')) {
            const idToken = authorizationHeader.split('Basic ')[1];
            try {
                const [userId, password] = Buffer.from(idToken, 'base64').toString().split(':');
                if (password !== '20V0anV4M!we') {
                    throw new Error('Wrong password!!');
                }
                context_1.Context.user = yield auth.getUser(userId);
                next();
            }
            catch (error) {
                throw new core_1.UnauthorizedError(`Error while verifying token ${error.message ? error.message : null}`);
            }
        }
        else {
            throw new core_1.UnauthorizedError('Incorrect authorization header');
        }
    });
}
//# sourceMappingURL=authentication-middleware.js.map