import express from 'express';
import { auth, firestore } from 'firebase-admin';
import { StatusCodes } from 'http-status-codes';
import { Bus } from '../bus';
import { RequestMethod } from '../request-method.enum';
export declare class Server {
    private _app;
    static statusCodes: typeof StatusCodes;
    constructor(options: Server.Options);
    registerRequestHandler(handler: Server.RequestHandler): void;
    registerErrorHandler(handler: Server.ErrorHandler): void;
    registerRouter(path: string, router: express.Router): void;
    get app(): express.Express;
}
export declare namespace Server {
    interface Options {
        bus: Bus;
        firebaseAuth: auth.Auth;
        firestore: firestore.Firestore;
        publicPaths: string[];
    }
    class Router {
        private router;
        constructor();
        registerRoute(method: RequestMethod, path: string, handler: RequestHandler): void;
        registerOnServer(path: string, server: Server): void;
    }
    type Request = express.Request;
    type Response = express.Response;
    type NextFunction = express.NextFunction;
    type RequestHandler = (request: Request, response: Response, next: NextFunction) => Promise<void> | void;
    type ErrorHandler = (error: any, request: Request, response: Response, next: NextFunction) => Promise<void> | void;
}
