"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || (function () {
    var ownKeys = function(o) {
        ownKeys = Object.getOwnPropertyNames || function (o) {
            var ar = [];
            for (var k in o) if (Object.prototype.hasOwnProperty.call(o, k)) ar[ar.length] = k;
            return ar;
        };
        return ownKeys(o);
    };
    return function (mod) {
        if (mod && mod.__esModule) return mod;
        var result = {};
        if (mod != null) for (var k = ownKeys(mod), i = 0; i < k.length; i++) if (k[i] !== "default") __createBinding(result, mod, k[i]);
        __setModuleDefault(result, mod);
        return result;
    };
})();
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Server = void 0;
const cors_1 = __importDefault(require("cors"));
const express_1 = __importDefault(require("express"));
const http_status_codes_1 = require("http-status-codes");
const request_method_enum_1 = require("../request-method.enum");
const middlewares = __importStar(require("./middlewares"));
function encapsulate(handler) {
    return (request, response, next) => __awaiter(this, void 0, void 0, function* () {
        try {
            yield handler(request, response, next);
        }
        catch (error) {
            next(error);
        }
    });
}
class Server {
    constructor(options) {
        this._app = (0, express_1.default)();
        this.registerRequestHandler((0, cors_1.default)({ origin: true }));
        this.registerRequestHandler(middlewares.CreateContextMiddleware(options.bus));
        this.registerRequestHandler(middlewares.HeadersMiddleWare());
        this.registerRequestHandler(middlewares.AuthenticationMiddleware(options.firebaseAuth, options.publicPaths));
    }
    registerRequestHandler(handler) {
        this._app.use(encapsulate(handler));
    }
    registerErrorHandler(handler) {
        this._app.use((error, request, response, next) => __awaiter(this, void 0, void 0, function* () {
            try {
                yield handler(error, request, response, next);
            }
            catch (error) {
                next(error);
            }
        }));
    }
    registerRouter(path, router) {
        this._app.use(path, router);
    }
    get app() {
        this.registerRequestHandler(middlewares.NotFoundMiddleware());
        this.registerErrorHandler(middlewares.ErrorMiddleWare());
        return this._app;
    }
}
exports.Server = Server;
Server.statusCodes = http_status_codes_1.StatusCodes;
(function (Server) {
    class Router {
        constructor() {
            this.router = express_1.default.Router();
        }
        registerRoute(method, path, handler) {
            switch (method) {
                case request_method_enum_1.RequestMethod.GET:
                    this.router.get(path, encapsulate(handler));
                    break;
                case request_method_enum_1.RequestMethod.POST:
                    this.router.post(path, encapsulate(handler));
                    break;
                default:
                    throw new Error(`Server.Router:registerRoute: ${method} is not managed yet...`);
            }
        }
        registerOnServer(path, server) {
            server.registerRouter(path, this.router);
        }
    }
    Server.Router = Router;
})(Server || (exports.Server = Server = {}));
//# sourceMappingURL=server.js.map