import { autoserializeAs, Document, DomainError, Event, guid, serialize } from '@my-devkit/core';

import { Context } from './context';

export abstract class Aggregate implements Document {
    @autoserializeAs(String) public id: string = null;
    @autoserializeAs(String) public _type: string = null;
    private _events: Event[] = [];

    public constructor() {
        this.id = guid();
        this._type = this.constructor.name;
    }

    public get _path(): string {
        return `/aggregates/${this.id}`;
    }

    public get hash(): string {
        return JSON.stringify(serialize(this));
    }

    protected assert(value: boolean, errorMessage: string): void {
        if (!value) {
            throw new DomainError(errorMessage);
        }
    }

    protected apply<E extends Event>(event: E): void {
        const methodName = `apply${event._type}`;
        if (!(methodName in this)) {
            throw new Error(`${this._type}.${methodName} method is missing!`);
        }

        event._aggregateId = this.id;
        event._aggregateType = this._type;

        event._correlationId = Context.correlationId;
        event._publishedBy = Context.user ? Context.user.uid : null;
        event.publishedAt = new Date();

        this[methodName](event);

        this._events.push(event);
    }

    public get events(): Event[] {
        return [...this._events];
    }

    public clearEvents(): void {
        this._events = [];
    }
}
