import { autoserializeAs } from '@my-devkit/core';
import { expect } from 'chai';

import { reflect } from '../../reflect';
import { Body } from './body';
import { Controller } from './controller';

describe('Given a Body decorator', () => {

    class Command {
        @autoserializeAs(String)
        public testString: string = null;

        @autoserializeAs(Date)
        public testDate: Date = null;
    }


    @Controller('test')
    class TestClass {
        public testDate(@Body() param1: Date): string {
            return `testDate: ${param1}`;
        }

        public testCommand(@Body() param1: Command): string {
            return `testCommand: ${param1}`;
        }
    }

    describe('When decorator is called', () => {
        it('Then we should retrieve information via reflect', async () => {
            const config = reflect(TestClass).getControllerConfiguration();
            expect(config.routes.length).equal(2, 'Wrong route count detected');

            const request1 = <any>{ body: '2021-02-08T11:14:23.318Z' };

            expect(config.routes[0].argumentInjectors.length).equal(1, 'Wrong argumentInjectors length');
            expect(config.routes[0].argumentInjectors[0](request1)).to.be.a('Date');
            expect(config.routes[0].argumentInjectors[0](request1).toISOString()).equals((new Date('2021-02-08T11:14:23.318Z')).toISOString());

            const request2 = <any>{ body: { testString: 'test' } };

            const expectedBody2 = new Command();
            expectedBody2.testString = 'test';
            expectedBody2.testDate = null;

            expect(config.routes[1].argumentInjectors.length).equal(1, 'Wrong argumentInjectors length');
            expect(config.routes[1].argumentInjectors[0](request2)).to.be.instanceOf(Command);
            expect(config.routes[1].argumentInjectors[0](request2)).deep.equal(expectedBody2);
        });
    });
});
