import { deserialize } from '@my-devkit/core';

import { reflect, RequestArgumentInjector } from '../../reflect';
import { Server } from '../../server';

export function Param(key: string): ParameterDecorator {
    return (target, propertyKey, parameterIndex) => {
        const methodArguments = reflect(target, propertyKey).getParameters();
        if (!methodArguments.hasOwnProperty(parameterIndex)) {
            throw new Error(`Param: can't find argument ${parameterIndex} on ${target.constructor.name}.${propertyKey.toString()}`);
        }

        const injector: RequestArgumentInjector = (req: Server.Request) => {
            if (req.params.hasOwnProperty(key)) {
                return deserialize(req.params[key], methodArguments[parameterIndex]);
            }
        };

        reflect(target, propertyKey).registerControllerRouteArgumentInjector(injector, parameterIndex);
    }
}
