import { autoserializeAs, guid } from '@my-devkit/core';
import { expect } from 'chai';

import { reflect } from '../../reflect';
import { RequestMethod } from '../../request-method.enum';
import { Body } from './body';
import { Controller } from './controller';
import { Post } from './post';

describe('Given a Post decorator', () => {

    class Command {
        @autoserializeAs(String)
        public testString: string = null;

        @autoserializeAs(Date)
        public testDate: Date = null;
    }


    @Controller('test')
    class TestClass {
        public test = guid();

        @Post('create')
        public createMethod(@Body() param1: Date): string {
            return `createMethod: ${param1}`;
        }

        @Post('update')
        public updateMethod(@Body() param1: Command): string {
            return `updateMethod: ${param1}`;
        }

        @Post('delete')
        public deleteMethod(@Body() param1: Command): string {
            return `deleteMethod: ${param1}`;
        }
    }

    describe('When decorator is called', () => {
        it('Then we should retrieve information via reflect', async () => {
            const config = reflect(TestClass).getControllerConfiguration();

            expect(config.routes.length).equal(3, 'Wrong route count detected');

            expect(config.routes[0].path).equal('create');
            expect(config.routes[0].methodName).equal('createMethod');
            expect(config.routes[0].requestMethod).equal(RequestMethod.POST);

            expect(config.routes[1].path).equal('update');
            expect(config.routes[1].methodName).equal('updateMethod');
            expect(config.routes[1].requestMethod).equal(RequestMethod.POST);

            expect(config.routes[2].path).equal('delete');
            expect(config.routes[2].methodName).equal('deleteMethod');
            expect(config.routes[2].requestMethod).equal(RequestMethod.POST);
        });
    });
});
