import { expect } from 'chai';

import { reflect } from '../../reflect';
import { Controller } from './controller';
import { Get } from './get';
import { Query } from './query';

describe('Given a Query decorator', () => {

    enum Gender {
        Male = 0,
        Female = 'Female'
    }

    @Controller('test')
    class TestClass {
        @Get('by-id')
        public getById(@Query('id') id: string): string {
            return `getById: ${id}`;
        }

        @Get('all')
        public all(@Query('minDate') minDate: Date, @Query('gender') gender: Gender): string {
            return `getByGenderOlderThan: ${gender} ${minDate.toISOString()}`;
        }
    }

    describe('When decorator is called', () => {
        it('Then we should retrieve information via reflect', async () => {
            const config = reflect(TestClass).getControllerConfiguration();
            expect(config.routes.length).equal(2, 'Wrong route count detected');

            const request1 = <any>{ query: { id: 'guid 1' } };
            expect(config.routes[0].argumentInjectors.length).equal(1, 'Wrong argumentInjectors length for getById');
            expect(config.routes[0].argumentInjectors[0](request1)).to.be.a('string');
            expect(config.routes[0].argumentInjectors[0](request1)).equals('guid 1');

            const request2 = <any>{ query: { gender: 0, minDate: '2021-02-15T14:23:00.318Z' } };

            expect(config.routes[1].argumentInjectors.length).equal(2, 'Wrong argumentInjectors length for getByGenderOlderThan');

            expect(config.routes[1].argumentInjectors[0](request2)).to.be.a('Date');
            expect(config.routes[1].argumentInjectors[0](request2).toISOString()).equals((new Date('2021-02-15T14:23:00.318Z')).toISOString());

            expect(config.routes[1].argumentInjectors[1](request2)).equals(Gender.Male);

            request2.query.gender = 'Female';
            expect(config.routes[1].argumentInjectors[1](request2)).equals(Gender.Female);
        });
    });
});
