import { Command } from '@my-devkit/core';

import { reflect } from '../../reflect';

export function CommandHandler(): MethodDecorator {
    return (target, propertyKey) => {
        const paramTypes = reflect(target, propertyKey).getParameters();

        if (paramTypes.length === 0) {
            throw new Error(`${target.constructor.name}.${propertyKey.toString()} should have one argument!`);
        }

        if (paramTypes.length > 1) {
            throw new Error(`${target.constructor.name}.${propertyKey.toString()} should have only one argument!`);
        }

        if (!(paramTypes[0].prototype instanceof Command)) {
            throw new Error(`${target.constructor.name}.${propertyKey.toString()} should have a command as argument!`);
        }

        reflect(target, propertyKey).registerCommandHandler(<any>paramTypes[0].prototype);
    }
}
