import { Event } from '@my-devkit/core';

import { ExecutionMode } from '../../execution-mode-enum';
import { reflect } from '../../reflect';

export function EventHandler(mode: ExecutionMode): MethodDecorator {
    return (target, propertyKey) => {
        const paramTypes = reflect(target, propertyKey).getParameters();

        if (paramTypes.length === 0) {
            throw new Error(`${target.constructor.name}.${propertyKey.toString()} should have one argument!`);
        }

        if (paramTypes.length > 1) {
            throw new Error(`${target.constructor.name}.${propertyKey.toString()} should have only one argument!`);
        }

        if (!(paramTypes[0].prototype instanceof Event)) {
            throw new Error(`${target.constructor.name}.${propertyKey.toString()} should have an event as argument!`);
        }

        reflect(target, propertyKey).registerEventHandler(<any>paramTypes[0].prototype, mode);
    }
}
