import { Type } from '../interfaces';

export interface ModuleOptions {
    /**
     * Optional list of imported modules that export the providers which are
     * required in this module.
     */
    imports?: Type[];
    /**
     * Optional list of controllers defined in this module which have to be
     * instantiated.
     */
    controllers?: Type[];

    /**
     * Optional list of handlers defined in this module which have to be instantiated.
     */
    handlers?: Type[];
}

export function Module(options: ModuleOptions): ClassDecorator {
    return target => {
        for (const key in options) {
            if (options.hasOwnProperty(key)) {
                Reflect.defineMetadata(key, options[key], target);
            }
        }
    };
}
