import { ITransactionalClient, Type } from './interfaces';
import { reflect } from './reflect';

export class Injector {
    private instances = new Map<string, any>();
    private transactionalClients: ITransactionalClient[] = [];

    public resolve<T>(target: Type<T>): T {
        const injectableId = reflect(target).getInjectableId();
        if (!injectableId) {
            throw new Error(`${target.name} is not injectable`);
        }

        if (!this.instances.has(injectableId)) {
            const tokens = reflect(target).getParameters();
            const injections = tokens.map(token => this.resolve<any>(token));
            const instance = new target(...injections);
            this.instances.set(injectableId, instance);

            if (reflect(target).isTransactionalClient()) {
                this.transactionalClients.push(<any>instance);
            }
        }

        return this.instances.get(injectableId);
    }

    public getTransactionalClients(): ITransactionalClient[] {
        return [...this.transactionalClients];
    }
}
