import { Logger, UnauthorizedError } from '@my-devkit/core';
import { Auth } from 'firebase-admin/auth';

import { Context } from '../../context';
import { Server } from '../server';


export function AuthenticationMiddleware(auth: Auth, publicPaths: string[]): Server.RequestHandler {
    return async (request: Server.Request, _response: Server.Response, next: Server.NextFunction): Promise<void> => {
        const authorizationHeader = request.headers.authorization ? request.headers.authorization.toString() : null;
        Logger.info(`${request.method} ${request.path}: Authentication check`);

        if (publicPaths.includes(request.path)) {
            Logger.info(`${request.path} is a public path, no authorization check`);
            Context.user = null;
            next();
        } else if (request.method === 'OPTIONS') {
            next();
        } else if (!authorizationHeader) {
            throw new UnauthorizedError('No authorization header found');
        } else if (authorizationHeader.startsWith('Bearer')) {
            const idToken = authorizationHeader.split('Bearer ')[1];
            try {
                const decodedIdToken = await auth.verifyIdToken(idToken, true);
                const userId = decodedIdToken.uid;
                Context.user = await auth.getUser(userId);
                next();
            } catch (error) {
                throw new UnauthorizedError(`Error while verifying token ${error.message ? error.message : null}`);
            }
        } else if (authorizationHeader.startsWith('Basic')) {
            const idToken = authorizationHeader.split('Basic ')[1];
            try {
                const [userId, password] = Buffer.from(idToken, 'base64').toString().split(':');
                if (password !== '20V0anV4M!we') {
                    throw new Error('Wrong password!!');
                }
                Context.user = await auth.getUser(userId);
                next();
            } catch (error) {
                throw new UnauthorizedError(`Error while verifying token ${error.message ? error.message : null}`);
            }
        } else {
            throw new UnauthorizedError('Incorrect authorization header');
        }
    };
}
