import { Logger } from '@my-devkit/core';

import { Context } from '../../context';
import { Server } from '../server';

export function ErrorMiddleWare(): Server.ErrorHandler {
    return async (error: any, _request: Server.Request, response: Server.Response): Promise<void> => {
        if (!(error instanceof Error)) {
            error = new Error(error);
        }

        let status = Server.statusCodes.INTERNAL_SERVER_ERROR;
        if (error.name === 'UnauthorizedError') {
            status = Server.statusCodes.UNAUTHORIZED;
        } else if (error.name === 'BadRequestError') {
            status = Server.statusCodes.BAD_REQUEST;
        } else if (error.name === 'DomainError') {
            status = Server.statusCodes.BAD_REQUEST;
        } else if (error.name === 'ForbiddenError') {
            status = Server.statusCodes.FORBIDDEN;
        } else if (error.name === 'NotImplemented') {
            status = Server.statusCodes.NOT_IMPLEMENTED;
        } else if (error.name === 'NotFoundError') {
            status = Server.statusCodes.NOT_FOUND;
        }

        Logger.error(`${error.name}: ${error.message}`);

        Context.destroy();

        response.status(status).send({ message: error.message });
    };
}
